@@ -86,8 +86,9 @@ Please specify something like '?charset=utf8' explicitly.""")
     @staticmethod
     def foreign_key(foreign_table_name, foreign_name = None, backref_name = None):
         """"Class decorator, add a foreign key to a SQLAlchemy model.
-        Parameter table_name is the destination table, in a one-to-many relationship, it is the "one" side.
-        foreign_name is the user-friendly name of table(if omitted, table will be used instead)
+        Parameters:
+        foreign_table_name is the destination table, in a one-to-many relationship, it is the "one" side.
+        foreign_name is the user-friendly name of table(if omitted, table will be used instead).
         backref_name is the name used to back ref the "many" side.
         """
         foreign_model_name = foreign_table_name.capitalize()
@@ -109,20 +110,23 @@ Please specify something like '?charset=utf8' explicitly.""")
         return ref_table
 
     @staticmethod
-    def many_to_many(ref_table_name, ref_name = None, backref_name = None):
+    def many_to_many(ref_table_name, ref_name = None, backref_name = None, middle_table_name = None):
         """Class Decorator, add a many-to-many relationship between two SQLAlchemy models.
-        Parameter table is the destination table, it is NOT the one which this method decorated on.
-        ref_name is how this model reference the destination models
+        Parameters:
+        ref_table_name is the destination table name, it is NOT the one which this method decorated on.
+        ref_name is how this model reference the destination models.
         backref_name is how the destination model reference this model.
+        middle_table_name is the middle table name of this many-to-many relationship.
         """
         ref_name = ref_name or '{0}s'.format(ref_table_name)
         ref_model_name = ref_table_name.capitalize()
         def ref_table(cls):
-            table_name = cls.__name__.lower()            
+            table_name = cls.__name__.lower()
+            my_middle_table_name = middle_table_name or '{0}_{1}'.format(table_name, ref_table_name)
             middle_table = Table(
-                '{0}_{1}'.format(table_name, ref_table_name), Database.Base.metadata,
-                Column('{0}_id'.format(table_name), Integer, ForeignKey('{0}.id'.format(table_name))),
-                Column('{0}_id'.format(ref_table_name), Integer, ForeignKey('{0}.id'.format(ref_table_name))),
+                my_middle_table_name, Database.Base.metadata,
+                Column('{0}_id'.format(table_name), Integer, ForeignKey('{0}.id'.format(table_name)), primary_key = True),
+                Column('{0}_id'.format(ref_table_name), Integer, ForeignKey('{0}.id'.format(ref_table_name)), primary_key = True),
             )
             my_backref_name = backref_name or '{0}s'.format(table_name)
             setattr(cls, ref_name, relationship(ref_model_name, secondary = middle_table, 
@@ -130,6 +134,29 @@ Please specify something like '?charset=utf8' explicitly.""")
             return cls
         return ref_table
 
+    @staticmethod
+    def self_many_to_many(ref_name, backref_name, middle_table_name):
+        """Class decorator, add a self-referential many-to-many relationship to a model.
+        Parameters:
+        ref_name and backref_name is how this model reference itself from two opposite direction
+        middle_table_name is the middle table name of this many-to-many relationship
+        """
+        def ref_table(cls):
+            model_name = cls.__name__
+            table_name = model_name.lower()
+            middle_table = Table(
+                middle_table_name, Database.Base.metadata,
+                # left follow right
+                Column('left_id', Integer, ForeignKey('{0}.id'.format(table_name)), primary_key = True),
+                Column('right_id', Integer, ForeignKey('{0}.id'.format(table_name)), primary_key = True)
+            )
+            setattr(cls, ref_name, relationship(model_name, secondary = middle_table, 
+                primaryjoin = cls.id == middle_table.c.left_id,
+                secondaryjoin = cls.id == middle_table.c.right_id,
+                lazy = 'dynamic', backref = backref(backref_name, lazy = 'dynamic')))
+            return cls
+        return ref_table
+
     @staticmethod
     def created_timestamp(cls):
         """Class decorator, add a timestamp field 'created' to the model"""
